/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This test shows how you can produce mixed INDEFINITE/DEFINITE encodings
 * using the DeferDecoding ASN.1 compiler directive.  A PDU component, c,
 * of type Bar affected by the DeferDecoding directive and is treated as
 * an OpenType value.  The keyword IMPLICIT is used for illustration.
 * The resulting tag sequence is: [4] IMPLICIT SEQUENCE value, i.e. [4] overlays
 * [UNIVERSAL 16].
 */


#include "defindef.h"

static char _v1[] = "Test";
static Bar _v0 = {
    _v1,
    TRUE
};

static S s1 = {
    25,
    TRUE,
    { 1, 0, NULL, &_v0}
};



int main(void)
{

    int		pdu_num, err = 0;
    OssBuf	encodedData;
    OssGlobal	w, *world = &w;
    void	*decodedData, *buf;


    err = ossinit(world, defindef);
    if (err) {
	printf("Initialization failed with error code = %d\n", err);
	return err;
    }

    ossPrint(world, "\tEncoding PDU Bar in DEFINITE-length form...\n\n");
    ossSetEncodingFlags(world, DEBUGPDU|DEFINITE);
    ossSetDecodingFlags(world, DEBUGPDU);
    encodedData.value = NULL;
    encodedData.length = 0;
    err = ossEncode(world, Bar_PDU, &_v0, &encodedData);
    if (err) {
	ossPrint(world, "%s\n", ossGetErrMsg(world));
	ossPrint(world, "Error code: %d\n", err);
	ossterm(world);
	return err;
    }
    ossPrintHex(world, (char *)encodedData.value, encodedData.length);

    s1.c.encoded = encodedData.value;
    s1.c.length = encodedData.length;

    encodedData.value = NULL;
    encodedData.length = 0;
    ossSetEncodingFlags(world, DEBUGPDU|INDEFINITE);
    ossSetDecodingFlags(world, DEBUGPDU);

    ossPrint(world, "\n\tUnencoded parent PDU S that includes PDU Bar...\n\n");
    ossPrintPDU(world, S_PDU, &s1);
    ossPrint(world, "\n\tEncoding PDU S in INDEFINITE-length form...\n\n");

    err = ossEncode(world, S_PDU, &s1, &encodedData);
    if (err) {
	ossPrint(world, "%s\n", ossGetErrMsg(world));
	ossPrint(world, "Error code: %d\n", err);
	ossFreeBuf(world, s1.c.encoded);
	ossterm(world);
	return err;
    }
    else {
	ossPrintHex(world, (char *)encodedData.value, encodedData.length);
	ossPrint(world, "\n\tDecoding parent PDU S...\n\n");
	buf  = encodedData.value;
	decodedData = NULL;
	pdu_num = S_PDU;
	err = ossDecode(world, &pdu_num, &encodedData, &decodedData);
	if (err) {
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	    ossPrint(world, "Error code: %d\n", err);
	    ossFreeBuf(world, buf);
	    ossterm(world);
	    return err;
	}
	else {
	    S *decodedDataSaved = decodedData;

	    ossPrint(world, "\n\tDecoded parent PDU S...\n\n");
	    ossPrintPDU(world, S_PDU, decodedData);
	    pdu_num = Bar_PDU;
	    encodedData.value = decodedDataSaved->c.encoded;
	    encodedData.length = decodedDataSaved->c.length;
	    decodedData = NULL;
	    ossPrint(world, "\n\tDecoding PDU Bar extracted from parent PDU S...\n\n");
	    err = ossDecode(world, &pdu_num, &encodedData, &decodedData);
	    if (err) {
		ossPrint(world, "%s\n", ossGetErrMsg(world));
		ossPrint(world, "Error code: %d\n", err);
	    }
	    else {
		ossPrint(world, "\n\tDecoded PDU Bar...\n\n");
		ossPrintPDU(world, pdu_num, decodedData);
		ossFreePDU(world, pdu_num, decodedData);
	    }
	    ossFreePDU(world, S_PDU, decodedDataSaved);
	    ossFreeBuf(world, buf);
	    ossFreeBuf(world, s1.c.encoded);
	    ossterm(world);
	    return 0;
	}
    }
}

